@@ -804,6 +804,8 @@ upload_photo_file_buffer_ready_cb (void     **buffer,
 		}
 
 		g_list_free (keys);
+		g_free (description);
+		g_free (title);
 		g_hash_table_unref (data_set);
 	}
 
