@@ -1363,6 +1363,8 @@ post_photo_file_buffer_ready_cb (void     **buffer,
 
 		g_free (tags);
 		g_list_free (keys);
+		g_free (description);
+		g_free (title);
 		g_hash_table_unref (data_set);
 	}
 
