@@ -35,6 +35,8 @@
 #define OVERLAY_MARGIN 10
 #define APPLY_ICC_PROFILE_BUTTON 4
 #undef ALWAYS_LOAD_ORIGINAL_SIZE
+#define N_FORWARD_PRELOADERS 2
+#define N_BACKWARD_PRELOADERS 2
 
 
 static void gth_viewer_page_interface_init (GthViewerPageInterface *iface);
@@ -100,6 +102,8 @@ struct _GthImageViewerPagePrivate {
 	gboolean           pointer_on_overview;
 	guint              hide_overview_id;
 	gboolean           apply_icc_profile;
+	GthFileData       *next_file_data[N_FORWARD_PRELOADERS];
+	GthFileData       *prev_file_data[N_BACKWARD_PRELOADERS];
 };
 
 
@@ -191,6 +195,44 @@ _gth_image_preloader_get_requested_size_for_current_image (GthImageViewerPage *s
 }
 
 
+static void
+_gth_image_viewer_page_load_with_preloader (GthImageViewerPage  *self,
+					    GthFileData         *file_data,
+					    int                  requested_size,
+					    GCancellable        *cancellable,
+					    GAsyncReadyCallback  callback,
+					    gpointer		 user_data)
+{
+	if (self->priv->apply_icc_profile)
+		gth_image_preloader_set_out_profile (self->priv->preloader, gth_browser_get_screen_profile (self->priv->browser));
+	else
+		gth_image_preloader_set_out_profile (self->priv->preloader, NULL);
+
+	g_object_ref (self);
+	gth_image_preloader_load (self->priv->preloader,
+				  file_data,
+				  requested_size,
+				  cancellable,
+				  callback,
+				  user_data,
+				  N_FORWARD_PRELOADERS + N_BACKWARD_PRELOADERS,
+				  self->priv->next_file_data[0],
+				  self->priv->next_file_data[1],
+				  self->priv->prev_file_data[0],
+				  self->priv->prev_file_data[1]);
+}
+
+
+static gboolean
+_gth_image_viewer_page_load_with_preloader_finish (GthImageViewerPage  *self)
+{
+	gboolean active = self->priv->active;
+	g_object_unref (self);
+
+	return active;
+}
+
+
 static void
 different_quality_ready_cb (GObject		*source_object,
 			    GAsyncResult	*result,
@@ -208,6 +250,9 @@ different_quality_ready_cb (GObject		*source_object,
 	int                 w1, h1, w2, h2;
 	gboolean            got_better_quality;
 
+	if (! _gth_image_viewer_page_load_with_preloader_finish (self))
+		return;
+
 	if (! gth_image_preloader_load_finish (GTH_IMAGE_PRELOADER (source_object),
 					       result,
 					       &requested,
@@ -288,16 +333,6 @@ _g_mime_type_can_load_different_quality (const char *mime_type)
 }
 
 
-static void
-_gth_image_preloader_init_preloader (GthImageViewerPage *self)
-{
-	if (self->priv->apply_icc_profile)
-		gth_image_preloader_set_out_profile (self->priv->preloader, gth_browser_get_screen_profile (self->priv->browser));
-	else
-		gth_image_preloader_set_out_profile (self->priv->preloader, NULL);
-}
-
-
 static gboolean
 update_quality_cb (gpointer user_data)
 {
@@ -308,21 +343,24 @@ update_quality_cb (gpointer user_data)
 		self->priv->update_quality_id = 0;
 	}
 
+	if (! self->priv->active)
+		return FALSE;
+
+	if (self->priv->viewer == NULL)
+		return FALSE;
+
 	if (self->priv->loading_image)
 		return FALSE;
 
 	if (! self->priv->image_changed && ! _g_mime_type_can_load_different_quality (gth_file_data_get_mime_type (self->priv->file_data)))
 		return FALSE;
 
-	_gth_image_preloader_init_preloader (self);
-	gth_image_preloader_load (self->priv->preloader,
-				  self->priv->image_changed ? GTH_MODIFIED_IMAGE : self->priv->file_data,
-				  _gth_image_preloader_get_requested_size_for_current_image (self),
-				  NULL,
-				  different_quality_ready_cb,
-				  self,
-				  GTH_NO_PRELOADERS,
-				  NULL);
+	_gth_image_viewer_page_load_with_preloader (self,
+						    self->priv->image_changed ? GTH_MODIFIED_IMAGE : self->priv->file_data,
+						    _gth_image_preloader_get_requested_size_for_current_image (self),
+						    NULL,
+						    different_quality_ready_cb,
+						    self);
 
 	return FALSE;
 }
@@ -547,7 +585,7 @@ viewer_realize_cb (GtkWidget *widget,
 	GthImageViewerPage *self = user_data;
 	GtkClipboard       *clipboard;
 
-	clipboard = gtk_widget_get_clipboard (self->priv->viewer, GDK_SELECTION_CLIPBOARD);
+	clipboard = gtk_widget_get_clipboard (widget, GDK_SELECTION_CLIPBOARD);
 	g_signal_connect (clipboard,
 	                  "owner_change",
 	                  G_CALLBACK (clipboard_owner_change_cb),
@@ -562,7 +600,7 @@ viewer_unrealize_cb (GtkWidget *widget,
 	GthImageViewerPage *self = user_data;
 	GtkClipboard       *clipboard;
 
-	clipboard = gtk_widget_get_clipboard (self->priv->viewer, GDK_SELECTION_CLIPBOARD);
+	clipboard = gtk_widget_get_clipboard (widget, GDK_SELECTION_CLIPBOARD);
 	g_signal_handlers_disconnect_by_func (clipboard,
 	                                      G_CALLBACK (clipboard_owner_change_cb),
 	                                      self);
@@ -635,6 +673,9 @@ pref_zoom_quality_changed (GSettings *settings,
 {
 	GthImageViewerPage *self = user_data;
 
+	if (! self->priv->active || (self->priv->viewer == NULL))
+		return;
+
 	gth_image_viewer_set_zoom_quality (GTH_IMAGE_VIEWER (self->priv->viewer),
 					   g_settings_get_enum (self->priv->settings, PREF_IMAGE_VIEWER_ZOOM_QUALITY));
 	gtk_widget_queue_draw (self->priv->viewer);
@@ -648,6 +689,9 @@ pref_zoom_change_changed (GSettings *settings,
 {
 	GthImageViewerPage *self = user_data;
 
+	if (! self->priv->active || (self->priv->viewer == NULL))
+		return;
+
 	gth_image_viewer_set_zoom_change (GTH_IMAGE_VIEWER (self->priv->viewer),
 					  g_settings_get_enum (self->priv->settings, PREF_IMAGE_VIEWER_ZOOM_CHANGE));
 	gtk_widget_queue_draw (self->priv->viewer);
@@ -661,6 +705,9 @@ pref_reset_scrollbars_changed (GSettings *settings,
 {
 	GthImageViewerPage *self = user_data;
 
+	if (! self->priv->active || (self->priv->viewer == NULL))
+		return;
+
 	gth_image_viewer_set_reset_scrollbars (GTH_IMAGE_VIEWER (self->priv->viewer),
 					       g_settings_get_boolean (self->priv->settings, PREF_IMAGE_VIEWER_RESET_SCROLLBARS));
 }
@@ -868,6 +915,7 @@ gth_image_viewer_page_real_activate (GthViewerPage *base,
 	self->priv->preloader = gth_browser_get_image_preloader (browser);
 
 	self->priv->viewer = gth_image_viewer_new ();
+	g_object_add_weak_pointer (G_OBJECT (self->priv->viewer), &self->priv->viewer);
 	gtk_widget_add_events (self->priv->viewer, GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK);
 	gth_image_viewer_page_reset_viewer_tool (self);
 	gtk_widget_show (self->priv->viewer);
@@ -1025,6 +1073,8 @@ preloader_load_ready_cb (GObject	*source_object,
 	GError		   *error = NULL;
 
 	self->priv->loading_image = FALSE;
+	if (! _gth_image_viewer_page_load_with_preloader_finish (self))
+		return;
 
 	if (! gth_image_preloader_load_finish (GTH_IMAGE_PRELOADER (source_object),
 					       result,
@@ -1090,10 +1140,6 @@ clear_data:
 }
 
 
-#define N_FORWARD_PRELOADERS 2
-#define N_BACKWARD_PRELOADERS 2
-
-
 static void
 _gth_image_viewer_page_load (GthImageViewerPage *self,
 		             GthFileData        *file_data)
@@ -1101,8 +1147,6 @@ _gth_image_viewer_page_load (GthImageViewerPage *self,
 	GthFileStore *file_store;
 	GtkTreeIter   iter;
 	int           i;
-	GthFileData  *next_file_data[N_FORWARD_PRELOADERS];
-	GthFileData  *prev_file_data[N_BACKWARD_PRELOADERS];
 
 	if (self->priv->file_data != file_data) {
 		_g_object_unref (self->priv->file_data);
@@ -1112,9 +1156,9 @@ _gth_image_viewer_page_load (GthImageViewerPage *self,
 	self->priv->loading_image = TRUE;
 
 	for (i = 0; i < N_FORWARD_PRELOADERS; i++)
-		next_file_data[i] = NULL;
+		_g_clear_object (&self->priv->next_file_data[i]);
 	for (i = 0; i < N_BACKWARD_PRELOADERS; i++)
-		prev_file_data[i] = NULL;
+		_g_clear_object (&self->priv->prev_file_data[i]);
 
 	file_store = gth_browser_get_file_store (self->priv->browser);
 	if (gth_file_store_find_visible (file_store, self->priv->file_data->file, &iter)) {
@@ -1124,36 +1168,29 @@ _gth_image_viewer_page_load (GthImageViewerPage *self,
 		for (i = 0; i < N_FORWARD_PRELOADERS; i++) {
 			if (! gth_file_store_get_next_visible (file_store, &next_iter))
 				break;
-			next_file_data[i] = gth_file_store_get_file (file_store, &next_iter);
+			self->priv->next_file_data[i] = g_object_ref (gth_file_store_get_file (file_store, &next_iter));
 		}
 
 		next_iter = iter;
 		for (i = 0; i < N_BACKWARD_PRELOADERS; i++) {
 			if (! gth_file_store_get_prev_visible (file_store, &next_iter))
 				break;
-			prev_file_data[i] = gth_file_store_get_file (file_store, &next_iter);
+			self->priv->prev_file_data[i] = g_object_ref (gth_file_store_get_file (file_store, &next_iter));
 		}
 
 		gth_image_viewer_set_void (GTH_IMAGE_VIEWER (self->priv->viewer));
 	}
 
-	_gth_image_preloader_init_preloader (self);
-	gth_image_preloader_load (self->priv->preloader,
-				  self->priv->file_data,
+	_gth_image_viewer_page_load_with_preloader (self,
+						    self->priv->file_data,
 #ifdef ALWAYS_LOAD_ORIGINAL_SIZE
-				  GTH_ORIGINAL_SIZE,
+						    GTH_ORIGINAL_SIZE,
 #else
-				  _gth_image_preloader_get_requested_size_for_next_images (self),
+						    _gth_image_preloader_get_requested_size_for_next_images (self),
 #endif
-				  NULL,
-				  preloader_load_ready_cb,
-				  self,
-				  N_FORWARD_PRELOADERS + N_BACKWARD_PRELOADERS,
-				  next_file_data[0],
-				  next_file_data[1],
-				  prev_file_data[0],
-				  prev_file_data[1]);
-
+						    NULL,
+						    preloader_load_ready_cb,
+						    self);
 }
 
 
@@ -1615,6 +1652,7 @@ static void
 gth_image_viewer_page_finalize (GObject *obj)
 {
 	GthImageViewerPage *self;
+	int                 i;
 
 	self = GTH_IMAGE_VIEWER_PAGE (obj);
 
@@ -1636,6 +1674,11 @@ gth_image_viewer_page_finalize (GObject *obj)
 	_g_object_unref (self->priv->file_data);
 	_g_object_unref (self->priv->last_loaded);
 
+	for (i = 0; i < N_FORWARD_PRELOADERS; i++)
+		_g_clear_object (&self->priv->next_file_data[i]);
+	for (i = 0; i < N_BACKWARD_PRELOADERS; i++)
+		_g_clear_object (&self->priv->prev_file_data[i]);
+
 	G_OBJECT_CLASS (gth_image_viewer_page_parent_class)->finalize (obj);
 }
 
@@ -1695,6 +1738,11 @@ gth_image_viewer_page_init (GthImageViewerPage *self)
 	self->priv->pointer_on_viewer = FALSE;
 	self->priv->hide_overview_id = 0;
 	self->priv->apply_icc_profile = TRUE;
+
+	for (i = 0; i < N_FORWARD_PRELOADERS; i++)
+		self->priv->next_file_data[i] = NULL;
+	for (i = 0; i < N_BACKWARD_PRELOADERS; i++)
+		self->priv->prev_file_data[i] = NULL;
 }
 
 
@@ -1922,6 +1970,7 @@ get_original_data_free (OriginalImageData *data)
 	if (data == NULL)
 		return;
 
+	_g_object_unref (data->viewer_page);
 	_g_object_unref (data->cancellable);
 	_g_object_unref (data->result);
 	g_free (data);
@@ -1934,8 +1983,13 @@ original_image_ready_cb (GObject	*source_object,
 			 gpointer	 user_data)
 {
 	OriginalImageData *data = user_data;
-	GthImage        *image = NULL;
-	GError          *error = NULL;
+	GthImage          *image = NULL;
+	GError            *error = NULL;
+
+	if (! _gth_image_viewer_page_load_with_preloader_finish (data->viewer_page)) {
+		get_original_data_free (data);
+		return;
+	}
 
 	if (! gth_image_preloader_load_finish (GTH_IMAGE_PRELOADER (source_object),
 					       result,
@@ -1967,22 +2021,32 @@ gth_image_viewer_page_get_original (GthImageViewerPage	 *self,
 	OriginalImageData *data;
 
 	data = get_original_data_new ();
-	data->viewer_page = self;
+	data->viewer_page = g_object_ref (self);
 	data->result = g_simple_async_result_new (G_OBJECT (self),
 						  ready_callback,
 						  user_data,
 						  gth_image_viewer_page_get_original);
 	data->cancellable = (cancellable != NULL) ? g_object_ref (cancellable) : g_cancellable_new ();
 
-	_gth_image_preloader_init_preloader (self);
-	gth_image_preloader_load (self->priv->preloader,
-				  self->priv->image_changed ? GTH_MODIFIED_IMAGE : self->priv->file_data,
-				  GTH_ORIGINAL_SIZE,
-				  data->cancellable,
-				  original_image_ready_cb,
-				  data,
-				  GTH_NO_PRELOADERS,
-				  NULL);
+	if (gth_image_viewer_is_animation (GTH_IMAGE_VIEWER (self->priv->viewer))) {
+		GthImage *image;
+
+		image = gth_image_new_for_surface (gth_image_viewer_get_current_image (GTH_IMAGE_VIEWER (self->priv->viewer)));
+		g_simple_async_result_set_op_res_gpointer (data->result,
+							   image,
+							   (GDestroyNotify) g_object_unref);
+		g_simple_async_result_complete_in_idle (data->result);
+
+		get_original_data_free (data);
+	}
+	else {
+		_gth_image_viewer_page_load_with_preloader (self,
+							    self->priv->image_changed ? GTH_MODIFIED_IMAGE : self->priv->file_data,
+							    GTH_ORIGINAL_SIZE,
+							    data->cancellable,
+							    original_image_ready_cb,
+							    data);
+	}
 }
 
 
